@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#   Copyright (C) 2012 Jan-Philip Gehrcke
+#   Copyright (C) 2012 - 2013 Jan-Philip Gehrcke
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -18,23 +18,17 @@
 .. todo::
 
     - Implement poll/peek on read end. (It's impossible to identify complete
-        messages in advance, but within the framework only complete messages
-        are sent.)
+      messages in advance, but within the framework only complete messages
+      are sent.)
     - Can gevent's FileObjectPosix be of any use?
-    - Review buffer-implementation, consider, buffer(), memoryview(), ..
+    - Review buffer-implementation, consider, buffer() and memoryview().
     - hub.cancel_wait() (cf. gevent sockets) in close instead of lock check?
-    - Ensure portability between Python 2 and 3
-    - put() timeout? Relevant in case of filled up pipes. However, put()
-        duration cannot be controlled if write is blocking *after* partial
-        msg write. Doesn't make sense I think.
-
-    - pipe implementation on Windows based on NamedPipes with overlapping IO
-        could give useful control:
-        - async IO, peeknamedpipe, ReadfileEx, SetCommTimeouts, CancelIO
-        - use pywin32 or http://docs.python.org/2/library/ctypes.html
-        - problem: circumvent blocking Win32 calls (hand control to gevent)
-        - async IO without blocking any thread: IOCP
-          http://msdn.microsoft.com/en-us/library/aa365198%28VS.85%29.aspx
+    - Work on portability between Python 2 and 3.
+    - put() timeout? Relevant in case of pipe being full. However, put()
+      duration cannot be controlled if write is blocking *after* partial
+      msg write. Doesn't make sense I think.
+    - Implementation on Windows based on NamedPipes with overlapping IO
+      could give useful control. Use libuv as backend instead of libev?
 """
 
 import os
@@ -131,8 +125,8 @@ def pipe():
 
 def start_process(target, args=(), kwargs={}, daemon=None, name=None):
     """Spawn child process and execute function ``target(*args, **kwargs)``.
-    Any existing :class:`gipc._GIPCHandle` can be handed over to the child process
-    via ``args`` and/or ``kwargs``.
+    Any existing :class:`gipc._GIPCHandle` can be handed over to the child
+    process via ``args`` and/or ``kwargs``.
 
     :arg target:
         Function to be called in child as ``target(*args, **kwargs)``.
@@ -156,12 +150,12 @@ def start_process(target, args=(), kwargs={}, daemon=None, name=None):
 
     Process creation is based on ``multiprocessing.Process()``. When working
     with gevent, instead of calling ``Process()`` directly, it is highly
-    recommended to create child processes via :func:`start_process`. It takes
+    recommended to start child processes via :func:`start_process`. It takes
     care of
 
+        - re-initializing gevent and libev's event loop in the child process.
         - closing dispensable file descriptors after child process creation.
         - proper file descriptor inheritance on Windows.
-        - re-initialization of the event loop in the child process.
         - providing cooperative process methods (such as ``join()``).
 
     Calling this method breaks ``os.waitpid()`` on Unix: spawning the first
@@ -200,13 +194,13 @@ def start_process(target, args=(), kwargs={}, daemon=None, name=None):
 
 
 def _child(target, all_handles, args, kwargs):
-    """Wrapper function that runs in child process. Sanitizes situation and
-    executes user-given function.
+    """Wrapper function that runs in child process. Resets gevent/libev state
+    and executes user-given function.
 
     After fork on POSIX-compliant systems, gevent's state is inherited by the
     child which may lead to undesired behavior, such as greenlets running in
-    both, the parent and the child. Therefore, on Unix, gevent's state is
-    entirely reset before running the user-given function.
+    both, the parent and the child. Therefore, on Unix, gevent's and libev's
+    state is reset before running the user-given function.
     """
     log.debug("_child start. target: `%s`" % target)
     # Restore global `_all_handles` (required on Win, does not harm elsewhere).
@@ -215,16 +209,23 @@ def _child(target, all_handles, args, kwargs):
     global _all_handles
     _all_handles = all_handles
     if not WINDOWS:
-        # `gevent.reinit` calls `libev.ev_loop_fork()`, which is designed to
-        # be called after fork.
+        # `gevent.reinit` calls `libev.ev_loop_fork()`, which reinitialises
+        # the kernel state for backends that have one. Must be called in the
+        # child before using further libev API.
         gevent.reinit()
+        log.debug("Delete current hub's threadpool.")
+        hub = gevent.get_hub()
+        del hub.threadpool
+        hub._threadpool = None
         # Destroy default event loop via `libev.ev_loop_destroy()` and delete
         # hub. This dumps all registered events and greenlets that have been
         # duplicated from the parent via fork().
-        gevent.get_hub().destroy(destroy_loop=True)
+        log.debug("Destroy hub and default loop.")
+        hub.destroy(destroy_loop=True)
         # Create a new hub and a new default event loop via
         # `libev.gevent_ev_default_loop`.
         h = gevent.get_hub(default=True)
+        log.debug("Created new hub and default event loop.")
         assert h.loop.default, 'Could not create libev default event loop.'
     allargs = itertools.chain(args, kwargs.values())
     childhandles = [a for a in allargs if isinstance(a, _GIPCHandle)]
@@ -242,7 +243,7 @@ def _child(target, all_handles, args, kwargs):
     target(*args, **kwargs)
     for h in childhandles:
         try:
-            # The user or a child of this child might already have closen it.
+            # The user or a child of this child might already have closed it.
             h.close()
         except GIPCClosed:
             pass
@@ -287,8 +288,7 @@ class _GProcess(multiprocessing.Process):
             self._returnevent = gevent.event.Event()
             # Start grabbing SIGCHLD in libev event loop.
             hub.loop.install_sigchld()
-            # Run new process (based on `fork()` on POSIX-compliant systems,
-            # and on `CreateProcess()` on Windows).
+            # Run new process (based on `fork()` on POSIX-compliant systems).
             super(_GProcess, self).start()
             # The occurrence of SIGCHLD is recorded asynchronously in libev.
             # This guarantees proper behaviour even if the child watcher is
@@ -300,7 +300,7 @@ class _GProcess(multiprocessing.Process):
 
         def _on_sigchld(self, watcher):
             watcher.stop()
-            # Status evaluation taken from multiprocessing.forking.
+            # Status evaluation taken from `multiprocessing.forking`.
             if os.WIFSIGNALED(watcher.rstatus):
                 self._popen.returncode = -os.WTERMSIG(watcher.rstatus)
             else:
