@@ -1,6 +1,6 @@
 ﻿# -*- coding: utf-8 -*-
 #
-#   Copyright (C) 2012 Jan-Philip Gehrcke
+#   Copyright (C) 2012 - 2013 Jan-Philip Gehrcke
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -407,7 +407,7 @@ class TestContextManager():
             set_all_handles([])
             raise Exception("Cleanup was not successful.")
 
-    def testt_all_handles_length(self):
+    def test_all_handles_length(self):
         r, w = pipe()
         assert len(get_all_handles()) == 2
         r.close()
@@ -540,7 +540,9 @@ class TestGetTimeout():
         assert False
 
 
-class TestUsecases():
+class TestSimpleUseCases():
+    """Tests reproducing basic usage scenarios of gipc.
+    """
     def teardown(self):
         if get_all_handles():
             for h in get_all_handles():
@@ -695,5 +697,50 @@ def usecase_child_c(reader, syncwriter):
     sys.exit(5)
 
 
+class TestComplexUseCases():
+    """Tests with increased complexity; involving other parts of gevent
+    and reproducing common usage scenarios.
+    """
+    def teardown(self):
+            if get_all_handles():
+                for h in get_all_handles():
+                    try:
+                        h.close()
+                        os.close(h._fd)
+                    except (OSError, GIPCError, TypeError):
+                        pass
+                set_all_handles([])
+                raise Exception("Cleanup was not successful.")
+
+    def test_getaddrinfo_mp(self):
+        """This test would make gevent's hub threadpool kill upon hub
+        destruction in child block forever. Gipc resolves this by killing
+        threadpool even harder.
+        """
+        import gevent.socket as socket
+        socket.getaddrinfo("localhost", 21)
+        p = start_process(target=child_test_getaddrinfo_mp)
+        p.join(timeout=1)
+        assert p.exitcode == 0
+
+    def test_threadpool_resolver_mp(self):
+        h = gevent.get_hub()
+        t = h.threadpool
+        r = h.resolver
+        p = start_process(target=child_test_threadpool_resolver_mp)
+        p.join(timeout=1)
+        assert p.exitcode == 0
+
+
+def child_test_threadpool_resolver_mp():
+    h = gevent.get_hub()
+    t = h.threadpool
+    r = h.resolver
+
+
+def child_test_getaddrinfo_mp():
+    return
+
+
 if __name__ == "__main__":
     pass
